// BoltzmannSampler.hpp: this file is part of the REGAL project.
//
// REGAL : Random and Exhaustive Generators for Automata - Library
//
// Copyright (C) 2007 Julien DAVID.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// The complete GNU General Public Licence Notice can be found as the
// `COPYING' file in the root directory.
//
//
#ifndef BOLTZMANNSAMPLER
#define BOLTZMANNSAMPLER

#include <math.h>
#include<time.h>
#include<stdlib.h>
#include "VerboseOption.hpp"
#include "plaws/NonZeroPoissonLaw.hpp"


#include <iostream>


namespace regal{
  
  using namespace plaws;

  class BoltzmannSampler{
  private:
    DiscreteProbabilityLaw * nzpl;
    long double dzetaK; 
    double p;
    int * partition; /*!< value returned by the function "sample". Allocated only once */
    int autSize; /*!< Number of state in the automaton */
    int alphaSize; /*!< Size of the alphabet */ 
    int rejectCounter;

    
    void computeDzetaK();
    
    
  public:
    int getRejectCounter(){return rejectCounter;}
    int * sample();
    
    /**
       Creates a Boltzmann Sampler 
       @param s is the number of states in the automaton
       @param k is the size of the alphabet
    */
    BoltzmannSampler(const int & s=2,const int & k=2){
      time_t t;
      partition=new int[s];
      autSize=s;
      alphaSize=k;
      computeDzetaK();
      nzpl=new NonZeroPoissonLaw(dzetaK);
      srand(time(&t));
      verbose("Creation of a Boltzmann Sampler");
    }
    
    /**
       Destroy a BoltzmannSampler
    */
    ~BoltzmannSampler(){
      delete [] partition;
      delete nzpl;
      verbose("Destruction of a Boltzmann Sampler");
    }
  };
  
}
  
#endif
  
