// TreeAutomaton.hpp: this file is part of the REGAL project.
//
// REGAL : Random and Exhaustive Generators for Automata - Library
//
// Copyright (C) 2007 Julien DAVID.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// The complete GNU General Public Licence Notice can be found as the
// `COPYING' file in the root directory.
//
//
#ifndef TREEAUTOMATON_HPP
#define TREEAUTOMATON_HPP


using namespace std;


namespace regal{

  template<typename StateLabel_t,typename Sigma>
  class TreeNode{
  private:
    bool final;
    StateLabel_t label;
    map<Sigma,TreeNode *> sons;
    
  public:

    void setFinal(const bool & value){
      final=value;
    }
    
    bool isFinal(){return final;}
    
    void deleteSon(const Sigma & letter){
      sons[letter]=NULL;
    }

    bool addSon(const TreeNode * s,const Sigma & letter){
      if(sons[letter]!=NULL)
	return false;
      sons[letter]=s;
      return true;
    }
    
    TreeNode(const StateLabel_t & lab,const bool & fin=false){
      label=l;
      final=fin;
    }

    virtual ~TreeNode(){}

  };

  template<typename StateLabel_t,typename Sigma>
  class TreeAutomaton: public AbstractAutomaton<StateLabel_t, Sigma> {
  private:
    map<int,TreeNode*> tree;
    int size;
    

    StateLabel_t createState(){
      tree[size]=new TreeNode(size);
      return size++;
    }
    

  public:
    
    
    
    int getSize(){return size;}
    void addTransition(const StateLabel_t & start,const StateLabel_t & end,const Sigma & value){
      tree[getIntegerValue(start)]->addSon(tree[getIntegerValue(end)],value);
    }
    bool isFinal(const StateLabel_t & sl){return tree[getIntegerValue(start)]->isFinal();}

    bool isInitial(const StateLabel_t & sl){return (getIntegerValue(sl)==0)?true:false;}
    
    void setStateAsFinal(const StateLabel_t & sl,const bool & value){
      tree[getIntegerValue(start)]->setFinal(value);
    }
    
    void setStateAsInitial(const StateLabel_t & sl,const bool & value){}

    bool deleteState(const StateLabel_t & sl){
      if(tree[getIntegerValue(start)]!=NULL)
	delete tree[getIntegerValue(start)];
      size--;
    }
    

  };

}



#endif
