//    This file is part of RoadColoringImpl.
//
//    Copyright (C) 2008 Florian SIKORA sikora@univ-mlv.fr
//    http://igm.univ-mlv.fr/~fsikora/index.php?p=r
//
//    RoadColoringImpl is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    RoadColoringImpl is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    The complete GNU General Public Licence Notice can be found as the
//    `COPYING' file in the root directory.


package fr.umlv.roadcoloring.graph;

import fr.umlv.roadcoloring.main.Tools;

/**
 * Algorithm for testing if a graph is aperiodic
 * 
 * method aperiodic() return the gcd of the circle
 * if this is 1, graph is aperiodic
 * 
 * @author fsikora
 *
 */

public class Aperiodic {
	private int p;	//period
	
	//colors for deep browsing
	private final byte color[];
	private static final byte GREEN = 0;
	private static final byte ORANGE  = 1;
	private static final byte RED = 2;
	
	private final int[] c;
	private final Graph g;
	
	/**
	 * constructor
	 * @param g graph to test
	 */
	public Aperiodic(Graph g) {
		p=0;
		c = new int[g.getSize()];
		color = new byte[g.getSize()];
		this.g = g;
	}
	
	/**
	 * perform the test
	 * @return the gcd of the circles (1 <-> isAperiodic)
	 */
	public int aperiodic() {
		int start = 0;
		
		for (int i=0 ; i<g.getSize() ; ++i) {
			color[i] = GREEN;
		}
		
		c[start] = 0;
		p = 0;
		
		//stronglyConnected, all node will be read
		explorationRec(start);
		
		return p;
	}
	
	/**
	 * Deep browsing from node
	 * @param node start node
	 */
	private void explorationRec(int node) {
		color[node] = ORANGE;
		
		actionSuccessor(node, g.getRed(node));
		actionSuccessor(node, g.getBlue(node));

		color[node] = RED;
	}
	
	/**
	 * perform action on the node and his sucessor
	 * @param node current node
	 * @param success sucessor (next)
	 */
	private void actionSuccessor(int node, int success) {
		if (color[success] == GREEN) {
			c[success] = c[node] + 1;
			explorationRec(success);
		}
		else {
			p = Tools.gcd(p, c[success] - c[node] - 1);
		}
	}
}
