//    This file is part of RoadColoringImpl.
//
//    Copyright (C) 2008 Florian SIKORA sikora@univ-mlv.fr
//    http://igm.univ-mlv.fr/~fsikora/index.php?p=r
//
//    RoadColoringImpl is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    RoadColoringImpl is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    The complete GNU General Public Licence Notice can be found as the
//    `COPYING' file in the root directory.


package fr.umlv.roadcoloring.graph;

import java.util.HashSet;

/**
 * States in the pair automata
 * 
 * @author fsikora
 *
 */

public class PairsState {
	private String wordFromStart = "";
	private final HashSet<Integer> nodeIn;	//nodes in the state
	
	/**
	 * New empty state
	 */
	public PairsState() {
		nodeIn = new HashSet<Integer>();
	}
	
	/**
	 * New state with a set of nodes and a word for going to this state
	 * @param set
	 * @param word
	 */
	public PairsState(HashSet<Integer> set, String word) {
		nodeIn = set;
		this.wordFromStart = word;
	}
	
	/**
	 * Add a node in the state
	 * @param node
	 */
	protected void addNode(int node) {
		nodeIn.add(node);
	}
	
	public String getWord() {
		return wordFromStart;
	}
	
	public HashSet<Integer> getNodes() {
		return nodeIn;
	}
	
	public boolean equals(HashSet<Integer> set) {
		if (set.size() != nodeIn.size()) return false;
		
		for (int i : set) {
			if (!nodeIn.contains(i)) {
				return false;
			}
		}
		return true;
	}
}
