//    This file is part of RoadColoringImpl.
//
//    Copyright (C) 2008 Florian SIKORA sikora@univ-mlv.fr
//    http://igm.univ-mlv.fr/~fsikora/index.php?p=r
//
//    RoadColoringImpl is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    RoadColoringImpl is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    The complete GNU General Public Licence Notice can be found as the
//    `COPYING' file in the root directory.


package fr.umlv.roadcoloring.graph;

import java.util.HashSet;
import java.util.LinkedList;

/**
 * 
 * usefull for check the strongly connected components of a graph
 *
 */


public class StronglyConnectedComponents {
	//previous nodes for each node
	private final PrevNodes prevNodes;
	//colors for deep browsing
	private final byte color[];
	private static final byte GREEN = 0;
	private static final byte ORANGE  = 1;
	private static final byte RED = 2;
	//list with node in inverted order in the first browse
	private final LinkedList<Integer> revlist;
	// number of the component for each node
	private final Integer components[];
	private final Graph g;

	/**
	 * Constructor
	 * @param g graph to check
	 */
	public StronglyConnectedComponents (Graph g) {
		this.g = g;
		color = new byte [g.getSize()];
		revlist = new LinkedList<Integer>();
		prevNodes = new PrevNodes(g, new HashSet<Integer>());

		components = new Integer[g.getSize()];
	}

	/**
	 * First deeply browse
	 * @param u node
	 */
	private void firstVisit(Integer u) {
		color[u] = ORANGE;

		//browse out edge from u
		if(color[g.getBlue(u)] == GREEN) {
			firstVisit(g.getBlue(u));
		}
		if(color[g.getRed(u)] == GREEN) {
			firstVisit(g.getRed(u));
		}
		color[u] = RED;
		
		revlist.addFirst(u);
	}
	
	/**
	 * Seconde deeply browsing
	 * @param u node
	 * @param w number of the component
	 */
	private void secondVisit(Integer u, Integer w) {
		components[u] = w;
		color[u] = ORANGE;

		for (Integer v : prevNodes.getFromNode(u)) {
			if (color[v] == GREEN)
				secondVisit(v, w);
		}

		color[u] = RED;
	}
	
	/**
	 * compute the strongly connected components
	 * 
	 * @return an array with the number of associated component for each node 
	 */
	public Integer[] components() {
		// first browsing
		for (int i = 0; i < color.length; i++)
			color[i] = GREEN;
		revlist.clear();
		for(int v = 0 ; v < g.getSize() ; ++v) {
			if (color[v] == GREEN)
				firstVisit(v);
		}

		// Second browsing
		for (int i = 0; i < color.length; i++)
			color[i] = GREEN;
		for (Integer v : revlist) {
			if (color[v] == GREEN)
				secondVisit(v, v);
		}
		return components;
	}
}
